@@ -1,7 +1,7 @@
 # SPDX-License-Identifier: MIT
-# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
-# Copyright (C) 2012-2014 Bastian Kleineidam
-# Copyright (C) 2015-2019 Tobias Gruetzmacher
+# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
+# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
+# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
 """
 Automated comic downloader. Dosage traverses comic websites in
 order to download each strip of the comic. The intended use is for
@@ -13,20 +13,18 @@ your intentions, and received permission to distribute.
 The primary interface is the 'dosage' commandline script.
 Comic modules for each comic are located in L{dosagelib.plugins}.
 """
+import logging
+from importlib import metadata
 
+from . import logext  # noqa: F401 - needed here to preload logging extensions
+
+logger = logging.getLogger(__name__)
+
+AppName = 'dosage'
 try:
-    from importlib.metadata import version, PackageNotFoundError
-except ImportError:
-    from importlib_metadata import version, PackageNotFoundError
-
-from .output import out
-
-AppName = u'dosage'
-try:
-    __version__ = version(AppName)  # PEP 396
-except PackageNotFoundError:
+    __version__ = metadata.version(AppName)  # PEP 396
+except metadata.PackageNotFoundError:
     # package is not installed
-    out.warn('{} is not installed, no version available.'
-        ' Use at least {!r} or {!r} to fix this.'.format(
-            AppName, 'pip install -e .', 'setup.py egg_info'))
+    logger.warning('%s is not installed, no version available.'
+        ' Use something like %r to fix this.', AppName, 'pip install -e .')
     __version__ = 'ERR.NOT.INSTALLED'
