@@ -1,12 +1,11 @@
 # SPDX-License-Identifier: MIT
-# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
-# Copyright (C) 2012-2014 Bastian Kleineidam
-# Copyright (C) 2015-2022 Tobias Gruetzmacher
-# Copyright (C) 2019-2020 Daniel Ring
-import os
-
-from ..scraper import ParserScraper
+# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
+# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
+# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
+# SPDX-FileCopyrightText: © 2019 Daniel Ring
+from .. import util
 from ..helpers import bounceStarter
+from ..scraper import ParserScraper
 
 XPATH_LINK = '//a[d:class("%s") and contains(text(), "%s")]'
 XPATH_IMG = '//div[d:class("comicnav")]//a[img[contains(@alt, "%s")]]'
@@ -58,8 +57,8 @@ class ComicFury(ParserScraper):
     def __init__(self, name, sub, lang=None, adult=False, endOfLife=False, segmented=False):
         super().__init__('ComicFury/' + name)
         self.prefix = name
-        self.url = 'https://%s.webcomic.ws/comics/' % sub
-        self.stripUrl = self.url + '%s'
+        self.url = f'https://{sub}.webcomic.ws/comics/'
+        self.stripUrl = self.url + '%s/'
         self.firstStripUrl = self.stripUrl % '1'
         if lang:
             self.lang = lang
@@ -73,25 +72,30 @@ class ComicFury(ParserScraper):
                 '//img[d:class("comicsegmentimage")]',
             )
 
-    def namer(self, image_url, page_url):
-        parts = page_url.split('/')
-        path, ext = os.path.splitext(image_url)
-        num = parts[-1]
-        return "%s_%s%s" % (self.prefix, num, ext)
+    def namer(self, imageUrl, pageUrl):
+        num = util.urlpathsplit(pageUrl)[-1]
+        if self.multipleImagesPerStrip:
+            page = self.getPage(pageUrl)
+            images = self.match(page, '//img[d:class("comicsegmentimage")]/@src')
+            if len(images) > 1:
+                imageIndex = images.index(imageUrl) + 1
+                return f"{self.prefix}_{num}-{imageIndex}"
+        return f"{self.prefix}_{num}"
 
     def shouldSkipUrl(self, url, data):
         """Skip pages without images."""
         # Videos on Underverse
-        return (data.xpath('//div[@id="comicimagewrap"]//video') and
-            not data.xpath('//div[@id="comicimagewrap"]//img'))
+        return (self.match(data, '//div[@id="comicimagewrap"]//video') and
+            not self.match(data, '//div[@id="comicimagewrap"]//img'))
 
     @classmethod
-    def getmodules(cls):  # noqa: Allowed to be long
+    def getmodules(cls):
         return (
-            # These were once in the list below, but fell out from the index...
+            # Cannot be found via search?!?
             cls('BadassologyByMichaelBay', 'strudelology'),
             cls('DandyAndCompany', 'dandyandcompany'),
             cls('DeadAtNight', 'deadnight'),
+            cls('Exiern', 'exiern'),
             cls('Shatterrealm', 'shatterrealm'),
 
             # do not edit anything below since these entries are generated from
@@ -135,7 +139,7 @@ class ComicFury(ParserScraper):
             cls('AgentBishop', 'agentbishop'),
             cls('AHappierKindOfSad', 'ahappierkindofsad'),
             cls('AlbinoBrothers', 'albinobros'),
-            cls('Alderwood', 'alderwood'),
+            cls('Alderwood', 'alderwood', segmented=True),
             cls('AlexanderAndLucasRebooted', 'alexanderandlucas'),
             cls('AliaTerra', 'alia-terra'),
             cls('AlienIrony', 'alien-irony'),
@@ -761,6 +765,7 @@ class ComicFury(ParserScraper):
             cls('PilgrimEnEspanol', 'pilgrimenespanol', 'es'),
             cls('PITCHBLACK', 'pitchblack'),
             cls('PlasticBulletsMayhemUnloaded', 'plasticbulletsmayhemunloaded'),
+            cls('PMDOnBorrowedTime', 'onborrowedtime'),
             cls('Poharex', 'poharex'),
             cls('PokemonWarpers', 'pokemonwarpers'),
             cls('PokmonOurStory', 'pokemonos'),
@@ -783,6 +788,7 @@ class ComicFury(ParserScraper):
             cls('Pulse', 'pulse'),
             cls('PureHavoc', 'pure-havoc'),
             cls('Queenie', 'queenie'),
+            cls('QueenieAdventure', 'queenieadventure', adult=True, segmented=True),
             cls('QuestCorporeal', 'questcorporeal'),
             cls('Rain', 'rain'),
             cls('RandomlyAssembled', 'randomlyassembled'),
@@ -1138,6 +1144,7 @@ class ComicFury(ParserScraper):
             cls('Xibalba', 'xibalba'),
             cls('Xit', 'x-it'),
             cls('YesterdayBound', 'yesterdaybound'),
+            cls('YetAnotherPMDComic', 'yapc'),
             cls('YouAreNow', 'yan'),
             cls('YOURCHOICE', 'yourchoice'),
             cls('ZackDragonbladeAndTheExcalites', 'zackdragonblade'),
