@@ -1,8 +1,8 @@
 # SPDX-License-Identifier: MIT
-# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
-# Copyright (C) 2012-2014 Bastian Kleineidam
-# Copyright (C) 2015-2022 Tobias Gruetzmacher
-# Copyright (C) 2019-2020 Daniel Ring
+# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
+# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
+# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
+# SPDX-FileCopyrightText: © 2019 Daniel Ring
 from ..scraper import Scraper
 
 
@@ -12,6 +12,7 @@ class Removed(Scraper):
                ' very complicated.',
         'del': 'Comic was removed from the web.',
         'block': 'The comic site is blocking us.',
+        'deny': "The comic doesn't want to be listed in Dosage (request from owner or robots.txt",
         'unk': 'Comic was removed for an unknown reason.',
         'brk': 'Comic navigation is broken.',
         'mov': 'Comic moved to a new hoster and no new module was written.',
@@ -28,7 +29,7 @@ class Removed(Scraper):
         return {'rem-' + self.reason: self.REASONS[self.reason]}
 
     @classmethod
-    def getmodules(cls):  # noqa: Allowed to be long
+    def getmodules(cls):
         return (
             # Removed in 2.16
             cls('AbleAndBaker'),
@@ -208,8 +209,7 @@ class Removed(Scraper):
             cls('Creators/GirlsandSportsinSpanish'),
             cls('Creators/Recess'),
             cls('Creators/RugratsinSpanish'),
-            cls('CtrlAltDel', 'block'),
-            cls('CtrlAltDel/Sillies', 'block'),
+            cls('CtrlAltDel/Sillies'),
             cls('DailyDose'),
             cls('DamnLol'),
             cls('DeathToTheExtremist'),
@@ -327,7 +327,6 @@ class Removed(Scraper):
             cls('GoComics/MinimumSecurity'),
             cls('GoComics/MixedMedications'),
             cls('GoComics/Molebashed'),
-            cls('GoComics/MollyandtheBear'),
             cls('GoComics/Mortimer'),
             cls('GoComics/MrGigiAndTheSquid'),
             cls('GoComics/MrMorris'),
@@ -1580,11 +1579,117 @@ class Removed(Scraper):
             cls('WebToons/Adamsville'),
             cls('WebToons/CrapIDrewOnMyLunchBreak'),
             cls('WintersLight'),
+
+            # Removed in 3.1
+            cls('AbbysAgency', 'brk'),
+            cls('AcademyVale'),
+            cls('AhoyEarth', 'block'),
+            cls('Anaria', 'del'),
+            cls('Angels2200', 'del'),
+            cls('BlackRose', 'brk'),
+            cls('BloodBound', 'deny'),
+            cls('CatenaManor/CatenaCafe'),
+            cls('ComicsKingdom/AmazingSpiderman'),
+            cls('ComicsKingdom/AmazingSpidermanSpanish'),
+            cls('ComicsKingdom/BigBenBoltSundays'),
+            cls('ComicsKingdom/BonersArkSundays'),
+            cls('ComicsKingdom/BrianDuffy'),
+            cls('ComicsKingdom/Crankshaft'),
+            cls('ComicsKingdom/FlashGordonSundays'),
+            cls('ComicsKingdom/FunkyWinkerbean'),
+            cls('ComicsKingdom/FunkyWinkerbeanSunday'),
+            cls('ComicsKingdom/FunkyWinkerbeanSundays'),
+            cls('ComicsKingdom/FunkyWinkerbeanVintage'),
+            cls('ComicsKingdom/HeartOfJulietJonesSundays'),
+            cls('ComicsKingdom/KatzenjammerKidsSundays'),
+            cls('ComicsKingdom/Lockhorns'),
+            cls('ComicsKingdom/MandrakeTheMagicianSundays'),
+            cls('ComicsKingdom/MikePeters'),
+            cls('ComicsKingdom/MotherGooseAndGrimm'),
+            cls('ComicsKingdom/PhantomSundays'),
+            cls('ComicsKingdom/PrinceValiantSundays'),
+            cls('ComicsKingdom/Retail'),
+            cls('ComicsKingdom/TigerSundays'),
+            cls('Everblue', 'block'),
+            cls('FalseStart'),
+            cls('FireflyCross'),
+            cls('Ginpu'),
+            cls('GoComics/9ChickweedLaneClassics'),
+            cls('GoComics/Badlands'),
+            cls('GoComics/BeanieTheBrownie'),
+            cls('GoComics/BearWithMe'),
+            cls('GoComics/BigNateFirstClass'),
+            cls('GoComics/BreakOfDay'),
+            cls('GoComics/Candorville'),
+            cls('GoComics/DilbertClassics'),
+            cls('GoComics/DilbertEnEspanol'),
+            cls('GoComics/DumbwichCastle'),
+            cls('GoComics/EyebeamClassic'),
+            cls('GoComics/GarfieldClassics'),
+            cls('GoComics/LaloAlcarazEnEspanol'),
+            cls('GoComics/MakingIt'),
+            cls('GoComics/MtPleasant'),
+            cls('GoComics/PCAndPixel'),
+            cls('GoComics/PetuniaAndDre'),
+            cls('GoComics/RosebudsEnEspanol'),
+            cls('GoComics/RudyPark'),
+            cls('GoComics/SaltNPepper'),
+            cls('GoComics/SigneWilkinson'),
+            cls('GoComics/Snowflakes'),
+            cls('GoComics/StoneSoupClassics'),
+            cls('GoComics/StuartCarlson'),
+            cls('GoComics/SunshineState'),
+            cls('GoComics/TruthFacts'),
+            cls('GoComics/Wannabe'),
+            cls('KemonoCafe/PrincessBunny'),
+            cls('Lackadaisy', 'block'),
+            cls('MangaDex/AttackOnTitan', 'legal'),
+            cls('MangaDex/DeliciousinDungeon', 'legal'),
+            cls('MangaDex/FuguushokuKajishiDakedoSaikyouDesu', 'legal'),
+            cls('MangaDex/ImTheMaxLevelNewbie', 'legal'),
+            cls('MangaDex/KaetteKudasaiAkutsuSan', 'legal'),
+            cls('MangaDex/PashiriNaBokuToKoisuruBanchouSan', 'legal'),
+            cls('MangaDex/PleaseDontBullyMeNagatoro', 'legal'),
+            cls('MangaDex/SaekiSanWaNemutteru', 'legal'),
+            cls('MangaDex/SoloLeveling', 'legal'),
+            cls('MangaDex/SousouNoFrieren', 'legal'),
+            cls('MangaDex/TenseiShitaraSlimeDattaKen', 'legal'),
+            cls('MangaDex/YuYuHakusho', 'legal'),
+            cls('MrLovenstein', 'jsh'),
+            cls('MyCartoons'),
+            cls('RickGriffinStudios/TracesOfThePastNSFW'),
+            cls('Shivae/BlackRose', 'brk'),
+            cls('SoloLeveling', 'legal'),
+            cls('StudioKhimera/Mousechevious'),
+            cls('TalesAndTactics'),
+            cls('Tapas/HoneyAndTheMoon'),
+            cls('TracesOfThePast/NSFW'),
+            cls('VampireHunterBoyfriends'),
+            cls('WebToons/ABudgiesLife'),
+            cls('WebToons/Anthronauts'),
+            cls('WebToons/AssassinRoommate'),
+            cls('WebToons/ChocoLatte'),
+            cls('WebToons/CrystalVirus'),
+            cls('WebToons/NightmareFactory'),
+            cls('WebToons/OVERPOWERED'),
+
+            # Removed in 3.2
+            cls('ComicsKingdom/BeetleMoses'),
+            cls('ComicsKingdom/BobMankoffPresentsShowMeTheFunny'),
+            cls('ComicsKingdom/BobMankoffPresentsShowMeTheFunnyAnimalEdition'),
+            cls('ComicsKingdom/Candorville'),
+            cls('ComicsKingdom/DarrinBell'),
+            cls('ComicsKingdom/FunnyOnlineAnimals'),
+            cls('ComicsKingdom/GodsHands'),
+            cls('ComicsKingdom/MaraLlaveKeeperOfTime'),
+            cls('Dilbert'),
+            cls('GoComics/DarrinBell'),
+            cls('GoComics/EverydayPeopleCartoons'),
         )
 
 
 class Renamed(Scraper):
-    MSG = 'Comic module was renamed to "%s", please rename the directory.'
+    MSG = 'Comic module was renamed/moved to "%s", please use the new name instead.'
     count = 0
 
     @classmethod
@@ -1601,7 +1706,7 @@ class Renamed(Scraper):
         return {'ren-%i' % self.i: self.MSG % self.newname}
 
     @classmethod
-    def getmodules(cls):  # noqa: Allowed to be long
+    def getmodules(cls):
         return (
             # Renamed in 2.16
             cls('1997', '1977'),
@@ -1658,28 +1763,43 @@ class Renamed(Scraper):
             # Renamed in 3.0
             cls('AHClub', 'RickGriffinStudios/AHClub'),
             cls('ComicFury/MuddlemarchMudCompany', 'ComicFury/MudCompany'),
-            cls('ComicsKingdom/FunkyWinkerbeanSundays', 'ComicsKingdom/FunkyWinkerbeanSunday'),
             cls('ComicsKingdom/ShermansLagoon', 'GoComics/ShermansLagoon'),
             cls('ComicsKingdom/TheLittleKing', 'ComicsKingdom/LittleKing'),
-            cls('ComicsKingdom/TigerSundays', 'ComicsKingdom/TigerVintageSundays'),
-            cls('GoComics/BloomCounty2017', 'GoComics/BloomCounty2019'),
-            cls('GoComics/Cathy', 'GoComics/CathyClassics'),
-            cls('GoComics/DarrinBell', 'ComicsKingdom/DarrinBell'),
+            cls('GoComics/BloomCounty2017', 'GoComics/BloomCountyContinues'),
             cls('GoComics/Owlturd', 'GoComics/ShenComix'),
-            cls('GoComics/PeanutsEnEspanol', 'GoComics/SnoopyEnEspanol'),
             cls('GoComics/RipleysBelieveItOrNotSpanish', 'GoComics/RipleysAunqueUstedNoLoCrea'),
             cls('GoComics/WebcomicName', 'WebcomicName'),
             cls('GoComics/Widdershins', 'Widdershins'),
             cls('Guardia', 'ComicFury/Guardia'),
             cls('RadioactivePanda', 'Tapas/RadioactivePanda'),
-            cls('Shivae/BlackRose', 'BlackRose'),
             cls('SmackJeeves/BlackTapestries', 'ComicFury/BlackTapestries'),
             cls('SmackJeeves/ByTheBook', 'ByTheBook'),
             cls('SmackJeeves/FurryExperience', 'ComicFury/FurryExperience'),
             cls('SmackJeeves/GrowingTroubles', 'ComicFury/GrowingTroubles'),
             cls('SmackJeeves/TheRealmOfKaerwyn', 'ComicFury/TheRealmOfKaerwyn'),
-            cls('SoloLeveling', 'MangaDex/SoloLeveling'),
             cls('StudioKhimera/Draconia', 'Draconia'),
+            cls('StudioKhimera/UberQuest', 'UberQuest'),
             cls('TracesOfThePast', 'RickGriffinStudios/TracesOfThePast'),
-            cls('TracesOfThePast/NSFW', 'RickGriffinStudios/TracesOfThePastNSFW'),
+
+            # Renamed in 3.1
+            cls('ComicsKingdom/SlylockFoxAndComicsForKids', 'ComicsKingdom/SlylockFox'),
+            cls('ComicsKingdom/SlylockFoxAndComicsForKidsSpanish',
+                'ComicsKingdom/SlylockFoxSpanish'),
+            cls('Derideal/LRE', 'Derideal/RLE'),
+            cls('Exiern', 'ComicFury/Exiern'),
+            cls('GoComics/FalseKnees', 'WebToons/FalseKnees'),
+            cls('GoComics/HeavenlyNostrils', 'GoComics/PhoebeAndHerUnicorn'),
+            cls('GoComics/Rosebuds', 'ComicsKingdom/Rosebuds'),
+            cls('MangaDex/TheWolfAndRedRidingHood', 'WebToons/TheWolfAndRedRidingHood'),
+            cls('MaxOveracts', 'OccasionalComicsDisorder'),
+            cls('SafelyEndangered', 'WebToons/SafelyEndangered'),
+
+            # Renamed in 3.2
+            cls('ComicsKingdom/WillyBlacksSpanish', 'ComicsKingdom/WillyBlackSpanish'),
+            cls('GoComics/BloomCounty2019', 'GoComics/BloomCountyContinues'),
+            cls('GoComics/CathyClassics', 'GoComics/Cathy'),
+            cls('GoComics/OutOfTheGenePoolReRuns', 'GoComics/OutOfTheGenePool'),
+            cls('GoComics/SnoopyEnEspanol', 'GoComics/PeanutsEnEspanol'),
+            cls('GoComics/TheWizardOfIdSpanish', 'GoComics/WizardOfIdEnEspanol'),
+            cls('PiledHigherAndDeeper', 'PHDComics'),
         )
