@@ -1,15 +1,21 @@
 # SPDX-License-Identifier: MIT
-# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
-# Copyright (C) 2012-2014 Bastian Kleineidam
-# Copyright (C) 2015-2022 Tobias Gruetzmacher
-# Copyright (C) 2019-2020 Daniel Ring
-from re import compile, escape, IGNORECASE, sub
-from os.path import splitext
+# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
+# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
+# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
+# SPDX-FileCopyrightText: © 2019 Daniel Ring
+from re import IGNORECASE, compile, escape, sub
 
-from ..scraper import _BasicScraper, _ParserScraper, ParserScraper
-from ..helpers import indirectStarter, bounceStarter, joinPathPartsNamer
+from .. import util
+from ..helpers import bounceStarter, indirectStarter, joinPathPartsNamer
+from ..scraper import ParserScraper, _BasicScraper, _ParserScraper
 from ..util import tagre
-from .common import ComicControlScraper, WordPressScraper, WordPressNavi, WordPressWebcomic
+from .common import (
+    ComicControlScraper,
+    WordPressNavi,
+    WordPressScraper,
+    WordPressSpliced,
+    WordPressWebcomic,
+)
 
 
 class SabrinaOnline(_BasicScraper):
@@ -33,11 +39,6 @@ class SabrinaOnline(_BasicScraper):
         return archivepages[-1]
 
 
-class SafelyEndangered(WordPressNavi):
-    url = 'http://www.safelyendangered.com/'
-    firstStripUrl = url + 'comic/ignored/'
-
-
 class SaffronAndSage(WordPressScraper):
     url = 'https://www.saffroncomic.com/'
     firstStripUrl = url + 'comic/p0001/'
@@ -74,19 +75,18 @@ class Savestate(WordPressNavi):
 
 
 class ScandinaviaAndTheWorld(_ParserScraper):
-    url = 'https://satwcomic.com/'
-    stripUrl = url + '%s'
-    firstStripUrl = stripUrl % 'sweden-denmark-and-norway'
+    url = 'https://satwcomic.com/sweden-denmark-and-norway'
+    firstStripUrl = url
     starter = indirectStarter
     imageSearch = '//img[@itemprop="image"]'
     prevSearch = '//a[@accesskey="p"]'
-    latestSearch = '//a[text()="View latest comic"]'
+    latestSearch = '//a[contains(@title, "Latest")]'
     textSearch = '//span[@itemprop="articleBody"]'
-    help = 'Index format: stripname'
 
 
 class ScaryGoRound(_ParserScraper):
-    url = 'http://www.scarygoround.com/sgr/ar.php'
+    url = ('https://web.archive.org/web/20190327203330/'
+        'https://www.scarygoround.com/sgr/ar.php')
     stripUrl = url + '?date=%s'
     firstStripUrl = stripUrl % '20020604'
     imageSearch = '//img[contains(@src, "/strips/")]'
@@ -113,8 +113,8 @@ class ScenesFromAMultiverse(_BasicScraper):
     help = 'Index format: yyyy/mm/dd/stripname'
 
 
-class SchlockMercenary(_ParserScraper):
-    url = 'http://www.schlockmercenary.com/'
+class SchlockMercenary(ParserScraper):
+    url = 'https://www.schlockmercenary.com/'
     stripUrl = url + '%s'
     firstStripUrl = stripUrl % '2000-06-12'
     imageSearch = '//div[@class="strip-image-wrapper"]/img'
@@ -175,10 +175,10 @@ class SexyLosers(_ParserScraper):
     latestSearch = '//a[@rel="bookmark"]'
     help = 'Index format: nnn'
     starter = indirectStarter
-    namer = joinPathPartsNamer((-2,), (-1,), '-')
+    namer = joinPathPartsNamer(pageparts=(-1,), imageparts=(-1,), joinchar='-')
 
 
-class ShadesOfGray(_ParserScraper):
+class ShadesOfGray(ParserScraper):
     url = 'https://www.theduckwebcomics.com/Shades_of_Gray/'
     stripUrl = url + '%s/'
     firstStripUrl = stripUrl % '4820502'
@@ -186,11 +186,9 @@ class ShadesOfGray(_ParserScraper):
     prevSearch = '//a[img[@class="arrow_prev"]]'
     nextSearch = '//a[img[@class="arrow_next"]]'
     starter = bounceStarter
+    namer = joinPathPartsNamer(pageparts=(-1,))
     endOfLife = True
 
-    def namer(self, imageUrl, pageUrl):
-        return pageUrl.rstrip('/').rsplit('/', 1)[-1]
-
 
 class Sharksplode(WordPressScraper):
     url = 'http://sharksplode.com/'
@@ -198,21 +196,70 @@ class Sharksplode(WordPressScraper):
     allow_errors = (403,)
 
 
-class Sheldon(_BasicScraper):
-    url = 'http://www.sheldoncomics.com/'
-    rurl = escape(url)
-    stripUrl = url + 'archive/%s.html'
-    firstStripUrl = stripUrl % '011130'
-    imageSearch = compile(tagre("img", "src", r'(http://cdn\.sheldoncomics\.com/strips/[^"]+)'))
-    prevSearch = compile(tagre("a", "href", r'(%sarchive/\d+\.html)' % rurl,
-                               after="sidenav-prev"))
-    help = 'Index format: yymmdd'
+class Sheldon(ParserScraper):
+    url = 'https://www.sheldoncomics.com/'
+    firstStripUrl = url + 'comic/well-who-is-this/'
+    imageSearch = '//div[@id="comic"]//img/@data-src-img'
+    prevSearch = '//a[img[d:class("left")]]'
 
 
-class Shifters(WordPressNavi):
-    url = 'http://shiftersonline.com/'
-    stripUrl = url + '%s/'
-    firstStripUrl = stripUrl % 'shifters-redux-promo'
+class Shifters(ParserScraper):
+    baseUrl = 'https://shiftersonline.com/'
+    url = baseUrl + 'series/shifters-redux/'
+    stripUrl = baseUrl + 'comic/%s/'
+    firstStripUrl = stripUrl % 'chapter-1-pg-1'
+    imageSearch = '//div[@id="spliced-comic"]//span[@class="default-lang"]//img'
+    prevSearch = '//a[@class="previous-comic"]'
+    latestSearch = '//div[@id="comic-archive-list"]//a'
+    starter = indirectStarter
+    namer = joinPathPartsNamer(pageparts=(-1,))
+
+
+class ShiftersOnGossamerWings(Shifters):
+    name = 'Shifters/OnGossamerWings'
+    baseUrl = 'https://shiftersonline.com/'
+    url = baseUrl + 'series/shifters-on-gossamer-wings/'
+    stripUrl = baseUrl + 'comic/%s/'
+    firstStripUrl = stripUrl % 'on-gossamer-wings-cover'
+
+
+class ShiftersTheBeastWithin(Shifters):
+    name = 'Shifters/TheBeastWithin'
+    baseUrl = 'https://shiftersonline.com/'
+    url = baseUrl + 'series/shifters-the-beast-within/'
+    stripUrl = baseUrl + 'comic/%s/'
+    firstStripUrl = stripUrl % 'awakenings-pg-1'
+    endOfLife = True
+
+    def namer(self, imageUrl, pageUrl):
+        filename = util.urlpathsplit(pageUrl)[-1]
+        if filename.startswith('the-company-of-dragons'):
+            filename = 'in-' + filename
+        # Prepend chapter number to filename
+        chapters = [
+            'awakenings',
+            'lifting-the-veil',
+            'tears-of-blood',
+            'on-the-lam',
+            'shades-of-intrigue',
+            'catfight',
+            'out-of-control',
+            'damage-control',
+            'wolfs-clothing',
+            'strange-dreams',
+            'blood-bonds',
+            'the-other-team',
+            'get-ferrah',
+            'the-price-of-power',
+            'dogfight',
+            'surfacing',
+            'in-the-company-of-dragons',
+            'filler',
+        ]
+        for chapter in chapters:
+            if filename.startswith(chapter):
+                filename = 'chapter-' + str(chapters.index(chapter) + 1) + '-' + filename
+        return filename
 
 
 class ShipInABottle(WordPressScraper):
@@ -223,17 +270,15 @@ class ShipInABottle(WordPressScraper):
     help = 'Index format: number'
 
 
-class Shortpacked(_ParserScraper):
-    url = 'http://www.shortpacked.com/index.php'
-    stripUrl = url + '?id=%s'
-    css = True
-    imageSearch = 'img#comic'
-    prevSearch = 'a.prev'
-    help = 'Index format: nnn'
+class Shortpacked(ComicControlScraper):
+    url = 'https://www.shortpacked.com/comic/'
+    firstStripUrl = url + 'just-a-toy-store'
 
 
-class ShotgunShuffle(WordPressScraper):
-    url = 'http://shotgunshuffle.com/'
+class ShotgunShuffle(WordPressSpliced):
+    # Currently down, use archive.org in the meantime (08-2023)
+    url = ('https://web.archive.org/web/20230131163842/'
+        'https://shotgunshuffle.com/')
     firstStripUrl = url + 'comic/pilot/'
 
 
@@ -272,9 +317,12 @@ class SixPackOfOtters(WordPressWebcomic):
 
 
 class SkinDeep(WordPressWebcomic):
-    url = 'http://www.skindeepcomic.com/'
+    url = 'https://www.skindeepcomic.com/'
     stripUrl = url + 'archive/%s/'
     firstStripUrl = stripUrl % 'issue-1-cover'
+    imageSearch = '//div[d:class("webcomic-image")]//noscript/img'
+    starter = bounceStarter
+    namer = joinPathPartsNamer(pageparts=(-1,))
 
 
 class SleeplessDomain(ComicControlScraper):
@@ -282,28 +330,22 @@ class SleeplessDomain(ComicControlScraper):
     stripUrl = url + 'comic/%s'
     firstStripUrl = stripUrl % 'chapter-1-cover'
     starter = bounceStarter
-
-    def namer(self, imageUrl, pageUrl):
-        return pageUrl.rsplit('/', 1)[-1] + '.' + imageUrl.rsplit('.', 1)[-1]
+    namer = joinPathPartsNamer(pageparts=(-1,))
 
 
 class SlightlyDamned(ComicControlScraper):
     url = 'http://www.sdamned.com/'
     firstStripUrl = url + 'comic/prologue'
+    starter = bounceStarter
 
     def namer(self, imageurl, pageurl):
         """Clean up mixed filename formats."""
-        filename = pageurl.rsplit('/', 1)[-1]
-        if filename == '':
-            filename = imageurl.rsplit('-', 1)[-1]
-        else:
-            filename = 'SD' + filename + '.' + imageurl.rsplit('.', 1)[-1]
-        return filename
+        return 'SD' + util.urlpathsplit(pageurl)[-1]
 
 
-class SluggyFreelance(_ParserScraper):
-    url = 'http://sluggy.com/'
-    stripUrl = 'http://archives.sluggy.com/book.php?chapter=%s'
+class SluggyFreelance(ParserScraper):
+    url = 'https://sluggy.com/'
+    stripUrl = 'https://archives.sluggy.com/book.php?chapter=%s'
     firstStripUrl = stripUrl % '1'
     imageSearch = '//div[d:class("comic_content")]/img/@data-src'
     prevSearch = '//div[d:class("previous")]/a'
@@ -314,7 +356,7 @@ class SluggyFreelance(_ParserScraper):
 
     def namer(self, imageurl, pageurl):
         # Remove random noise from filename
-        return imageurl.rsplit('/', 1)[-1].split('.pagespeed', 1)[0]
+        return util.urlpathsplit(imageurl)[-1].split('.', 1)[0]
 
 
 class SMBC(ComicControlScraper):
@@ -325,9 +367,9 @@ class SMBC(ComicControlScraper):
     textSearch = '//img[@id="cc-comic"]/@title'
     multipleImagesPerStrip = True
 
-    def namer(self, imageUrl, pageUrl):
+    def namer(self, image_url, page_url):
         # Remove random noise from filename
-        filename = imageUrl.rsplit('/', 1)[-1]
+        filename = util.urlpathsplit(image_url)[-1]
         if '-' in filename and len(filename.rsplit('-', 1)[-1]) > 12:
             filename = filename.rsplit('-', 1)[-1]
         elif len(filename) > 22 and filename[0] == '1':
@@ -348,7 +390,7 @@ class SnowFlame(WordPressScraper):
         return self.stripUrl % tuple(index.split('-'))
 
     def namer(self, image_url, page_url):
-        prefix, filename = image_url.rsplit('/', 1)
+        filename = util.urlpathsplit(image_url)[-1]
         ro = compile(r'snowflame-([^-]+)-([^-]+)')
         mo = ro.search(page_url)
         chapter = mo.group(1)
@@ -381,28 +423,30 @@ class Sorcery101(WordPressWebcomic):
     help = 'Index format: stripname'
 
 
-class SpaceFurries(_ParserScraper):
-    url = 'http://www.spacefurrs.org/'
+class SpaceFurries(ParserScraper):
+    url = 'https://www.spacefurrs.org/'
     firstStripUrl = url
     multipleImagesPerStrip = True
     adult = True
     endOfLife = True
 
-    def fetchUrls(self, url, data, urlSearch):
+    def extract_image_urls(self, url, data):
         # Website requires JS, so build the list of image URLs manually
-        imageUrls = []
-        currentPage = int(data.xpath('//input[@name="pagnum"]')[0].get('value'))
-        for page in reversed(range(1, currentPage + 1)):
-            imageUrls.append(self.url + 'comics/' + str(page) + '.jpg')
-        return imageUrls
+        imageurls = []
+        current = int(self.match(data, '//input[@name="pagnum"]')[0].get('value'))
+        for page in reversed(range(1, current + 1)):
+            imageurls.append(self.url + 'comics/' + str(page) + '.jpg')
+        return imageurls
 
 
-class SpaceJunkArlia(_ParserScraper):
-    url = 'http://spacejunkarlia.com/'
+class SpaceJunkArlia(ParserScraper):
+    url = ('https://web.archive.org/web/20220121133701/'
+        'http://spacejunkarlia.com/')
     stripUrl = url + '?strip_id=%s'
     firstStripUrl = stripUrl % '0'
     imageSearch = '//div[d:class("content")]/img'
     prevSearch = '//a[text()="<"]'
+    endOfLife = True
     help = 'Index format: number'
 
 
@@ -499,12 +543,12 @@ class StandStillStaySilent(_ParserScraper):
     imageSearch = '//img[@class="comicnormal"]'
     prevSearch = '//a[./img[contains(@src, "nav_prev")]]'
 
-    def namer(self, imageUrl, pageUrl):
-        chapter = '2' if ('adv2_comicpages' in imageUrl) else '1'
-        return '%s-%s' % (chapter, imageUrl.rsplit('/', 1)[-1].replace('page_', ''))
+    def namer(self, image_url, page_url):
+        chapter = '2' if ('adv2_comicpages' in image_url) else '1'
+        return '%s-%s' % (chapter, util.urlpathsplit(image_url)[-1].replace('page_', ''))
 
 
-class StarCrossdDestiny(_ParserScraper):
+class StarCrossdDestiny(ParserScraper):
     baseUrl = ('https://web.archive.org/web/20190918132321/'
         'http://starcrossd.net/')
     url = baseUrl + 'comic.html'
@@ -522,35 +566,20 @@ class StarCrossdDestiny(_ParserScraper):
             image_url = sub('(?:strips)|(?:images)', 'book1', image_url)
         elif not image_url.find('strips') == -1:
             image_url = image_url.replace('strips/', '')
-        directory, filename = image_url.split('/')[-2:]
-        filename, extension = splitext(filename)
+        directory, filename = util.urlpathsplit(image_url)[-2:]
         return directory + '-' + filename
 
 
-class StarfireAgency(WordPressWebcomic):
-    url = 'https://poecatcomix.com/starfire-agency-static/'
-    stripUrl = 'https://poecatcomix.com/starfire-agency/%s/'
-    firstStripUrl = stripUrl % '2005-09-201'
-    imageSearch = '//div[contains(@class, "webcomic-media")]//img'
-
-    def starter(self):
-        # Build list of chapters for naming
-        indexPage = self.getPage(self.url)
-        self.chapters = indexPage.xpath('//a[./img[contains(@class, "attachment-large")]]/@href')
-        latestPage = self.chapters[0]
-        self.chapters = self.chapters[1:]
-        self.currentChapter = len(self.chapters)
-        return latestPage
-
-    def namer(self, imageUrl, pageUrl):
-        page = pageUrl.rstrip('/').rsplit('/', 1)[-1]
-        page = page.replace('3page00', 'cover3').replace('6429', 'cover7').replace('sfa-6-5-cover', 'cover6')
-        page = page.replace('sfa01', 'page01').replace('sfa03', 'page03').replace('sfa04', 'page04')
-        page = page.replace('sfa24', 'page24').replace('sfa07', 'page')
-        filename = 'sfa%d-%s.%s' % (self.currentChapter, page, imageUrl.rsplit('.', 1)[-1])
-        if pageUrl in self.chapters:
-            self.currentChapter = self.currentChapter - 1
-        return filename
+class StarfireAgency(ParserScraper):
+    url = 'https://poecatcomix.com/starfirecomic/'
+    stripUrl = url + '%s/'
+    firstStripUrl = stripUrl % 'sfa-issue-1-cover'
+    imageSearch = '//img[@class="scale-with-grid wp-post-image"]'
+    prevSearch = '//a[d:class("fixed-nav-prev")]'
+    latestSearch = '//div[@class="post-title"]//a'
+    starter = indirectStarter
+    adult = True
+    namer = joinPathPartsNamer(pageparts=(-1,))
 
 
 class StarTrip(ComicControlScraper):
@@ -581,12 +610,12 @@ class StringTheory(WordPressNavi):
     firstStripUrl = url + 'comics/chapterone/chapterone/'
 
 
-class StrongFemaleProtagonist(_ParserScraper):
-    url = 'http://strongfemaleprotagonist.com/'
+class StrongFemaleProtagonist(ParserScraper):
+    url = 'https://strongfemaleprotagonist.com/'
     stripUrl = url + '%s/'
-    css = True
-    imageSearch = 'article p img'
-    prevSearch = 'a.page-nav__item--left'
+    firstStripUrl = stripUrl % 'issue-1/page-0'
+    imageSearch = '//article/p/img'
+    prevSearch = '//a[d:class("page-nav__item--left")]'
     help = 'Index format: issue-?/page-??'
 
     def shouldSkipUrl(self, url, data):
@@ -602,16 +631,17 @@ class StrongFemaleProtagonist(_ParserScraper):
         )
 
 
-class StupidFox(_ParserScraper):
+class StupidFox(ParserScraper):
     url = 'http://stupidfox.net/'
     stripUrl = url + '%s'
     firstStripUrl = stripUrl % 'hello'
-    imageSearch = '//div[@class="comicmid"]//img'
+    imageSearch = '//div[d:class("comicmid")]//img'
     prevSearch = '//a[@accesskey="p"]'
 
     def namer(self, imageUrl, pageUrl):
         page = self.getPage(pageUrl)
-        title = page.xpath(self.imageSearch + '/@title')[0].replace(' - ', '-').replace(' ', '-')
+        title = self.match(page, self.imageSearch + '/@title')[0].replace(' - ',
+            '-').replace(' ', '-')
         return title + '.' + imageUrl.rsplit('.', 1)[-1]
 
 
@@ -666,3 +696,12 @@ class SwordsAndSausages(_ParserScraper):
     imageSearch = '//img[d:class("comic-image")]'
     prevSearch = '//a[./span[contains(text(), "Previous")]]'
     multipleImagesPerStrip = True
+
+
+class SwordsComic(ParserScraper):
+    url = 'https://swordscomic.com/'
+    stripUrl = url + 'comic/%s'
+    firstStripUrl = stripUrl % 'cover'
+    imageSearch = '//div[@class="page-image-wrapper"]//img'
+    prevSearch = '//a[@class="navigation-button navigation-previous"]'
+    help = 'Index format: Swordsnnn (unpadded)'
