@@ -1,6 +1,6 @@
 # SPDX-License-Identifier: MIT
-# Copyright (C) 2019-2022 Tobias Gruetzmacher
-# Copyright (C) 2019-2022 Daniel Ring
+# SPDX-FileCopyrightText: © 2019 Tobias Gruetzmacher
+# SPDX-FileCopyrightText: © 2019 Daniel Ring
 from ..scraper import ParserScraper
 
 
@@ -9,8 +9,11 @@ class WebToons(ParserScraper):
     prevSearch = '//a[contains(@class, "_prevEpisode")]'
     multipleImagesPerStrip = True
 
-    def __init__(self, name: str, path: str, titlenum: int):
+    def __init__(self, name: str, path: str, titlenum: int) -> None:
         super().__init__('WebToons/' + name)
+        # Since pages on WebToon are composed of many images, we can probably
+        # move a lot faster then default
+        self.session.add_throttle('webtoon-phinf.pstatic.net', 0.0, 0.05)
 
         baseUrl = 'https://www.webtoons.com/en/'
         self.url = baseUrl + path + '/episode/viewer?title_no=' + str(titlenum)
@@ -18,44 +21,45 @@ class WebToons(ParserScraper):
         self.stripUrl = self.url + '&episode_no=%s'
         self.firstStripUrl = self.stripUrl % '1'
 
-    def starter(self):
+    def starter(self) -> str:
         # Avoid age/GDPR gate
         for cookie in ('needGDPR', 'needCCPA', 'needCOPPA'):
             self.session.cookies.set(cookie, 'false', domain='webtoons.com')
         # Find current episode number
         listPage = self.getPage(self.listUrl)
-        currentEpisode = listPage.xpath('//div[@class="detail_lst"]/ul/li')[0].attrib['data-episode-no']
+        currentEpisode = self.match(listPage, '//div[d:class("detail_lst")]/ul/li')[0].attrib['data-episode-no']
         # Check for completed tag
-        self.endOfLife = (listPage.xpath('//div[@id="_asideDetail"]//span[@class="txt_ico_completed2"]') != [])
+        self.endOfLife = not self.match(listPage, '//div[@id="_asideDetail"]//span[d:class("txt_ico_completed2")]')
         return self.stripUrl % currentEpisode
 
-    def fetchUrls(self, url, data, urlSearch):
+    def extract_image_urls(self, url, data):
         # Save link order for position-based filenames
-        self.imageUrls = super().fetchUrls(url, data, urlSearch)
+        self._cached_image_urls = super().extract_image_urls(url, data)
         # Update firstStripUrl with the correct episode title
         if url.rsplit('=', 1)[-1] == '1':
             self.firstStripUrl = url
-        return self.imageUrls
+        return self._cached_image_urls
 
     def namer(self, imageUrl, pageUrl):
         # Construct filename from episode number and image position on page
         episodeNum = pageUrl.rsplit('=', 1)[-1]
-        imageNum = self.imageUrls.index(imageUrl)
+        imageNum = self._cached_image_urls.index(imageUrl)
         imageExt = pageUrl.rsplit('.', 1)[-1].split('?', 1)[0]
         return "%s-%03d.%s" % (episodeNum, imageNum, imageExt)
 
     @classmethod
-    def getmodules(cls):  # noqa: Allowed to be long
+    def getmodules(cls):
         return (
             cls('1000', 'action/one-thousand', 1217),
             cls('10thDimensionBoys', 'comedy/10th-dimension-boys', 71),
             cls('1111Animals', 'comedy/1111-animals', 437),
             cls('2015SpaceSeries', 'sf/2015-space-series', 391),
             cls('3SecondStrip', 'comedy/3-second-strip', 380),
+            cls('99ReinforcedStick', 'comedy/99-reinforced-wooden-stick', 4286),
             cls('ABittersweetLife', 'slice-of-life/a-bittersweet-life', 294),
             cls('AboutDeath', 'drama/about-death', 82),
-            cls('ABudgiesLife', 'slice-of-life/its-a-budgies-life', 985),
             cls('Acception', 'drama/acception', 1513),
+            cls('AcesWild', 'challenge/aces-wild', 689025),
             cls('Acursian', 'supernatural/acursian', 1452),
             cls('AdventuresOfGod', 'comedy/adventures-of-god', 853),
             cls('AerialMagic', 'fantasy/aerial-magic', 1358),
@@ -63,18 +67,17 @@ class WebToons(ParserScraper):
             cls('AGoodDayToBeADog', 'romance/a-good-day-tobe-a-dog', 1390),
             cls('Aisopos', 'drama/aisopos', 76),
             cls('AliceElise', 'fantasy/alice-elise', 1481),
+            cls('AlloyComics', 'canvas/alloy-comics', 747447),
             cls('AllThatWeHopeToBe', 'slice-of-life/all-that-we-hope-to-be', 470),
             cls('AllThatYouAre', 'drama/all-that-you-are', 403),
             cls('AlwaysHuman', 'romance/always-human', 557),
             cls('AMPED', 'challenge/amped', 532009),
             cls('Annarasumanara', 'drama/annarasumanara', 77),
-            cls('Anthronauts', 'challenge/anthronauts', 358917),
             cls('AphroditeIX', 'sf/aphroditeix', 1451),
             cls('ApocalypticHorseplay', 'supernatural/apocalyptic-horseplay', 635),
             cls('AprilFlowers', 'fantasy/april-flowers', 1363),
             cls('Arma', 'super-hero/arma', 1640),
             cls('AsPerUsual', 'slice-of-life/as-per-usual', 599),
-            cls('AssassinRoommate', 'romance/assassin-roommate', 1050),
             cls('AthenaComplex', 'fantasy/athena-complex', 867),
             cls('AuraFromAnotherPlanet', 'comedy/aura-from-another-planet', 369),
             cls('AverageAdventuresOfAnAverageGirl', 'slice-of-life/average-adventures-of-an-average-girl', 401),
@@ -86,6 +89,7 @@ class WebToons(ParserScraper):
             cls('BehindTheGIFs', 'comedy/behind-the-gifs', 658),
             cls('BigJo', 'romance/big-jo', 854),
             cls('BiteMe', 'thriller/bite-me', 1019),
+            cls('BitterSweetCoffee', 'challenge/bitter-sweet-coffee', 797203),
             cls('Blackened', 'challenge/blackened', 363805),
             cls('BladesOfFurry', 'romance/blades-of-furry', 2383),
             cls('Blessed', 'drama/blessed', 1193),
@@ -104,26 +108,29 @@ class WebToons(ParserScraper):
             cls('CARL', 'slice-of-life/carl', 1216),
             cls('Caster', 'action/caster', 1461),
             cls('CastleSwimmer', 'fantasy/castle-swimmer', 1499),
+            cls('CatchMeIfYouCan', 'challenge/catch-me-if-you-can-', 434808),
             cls('Catharsis', 'fantasy/catharsis', 396),
             cls('CatLoafAdventures', 'slice-of-life/cat-loaf-adventures', 1381),
             cls('CChansACatgirl', 'challenge/c-chans-a-catgirl', 263430),
             cls('CheeseInTheTrap', 'drama/cheese-in-the-trap', 99),
             cls('CherryBlossoms', 'romance/cherry-blossoms', 1005),
             cls('Chiller', 'thriller/chiller', 536),
-            cls('ChocoLatte', 'romance/choco-latte', 1691),
             cls('CityOfBlank', 'sf/city-of-blank', 1895),
             cls('CityOfWalls', 'drama/city-of-wall', 505),
             cls('CityVamps', 'challenge/city-vamps-', 119224),
             cls('ClawShot', 'challenge/clawshot', 621465),
+            cls('ClinicOfHorrors', 'supernatural/clinic-of-horrors', 3414),
             cls('ClusterFudge', 'slice-of-life/cluster-fudge', 355),
             cls('CodeAdam', 'action/code-adam', 1657),
             cls('CookingComically', 'tiptoon/cooking-comically', 622),
+            cls('CrowTime', 'challenge/crow-time', 693372),
             cls('Crumbs', 'romance/crumbs', 1648),
-            cls('CrystalVirus', 'challenge/crystal-virus', 347038),
+            cls('CrystalVirusOtherStory', 'challenge/crystal-virus-other-story', 837028),
             cls('CupidsArrows', 'romance/cupids-arrows', 1538),
             cls('CursedPrincessClub', 'comedy/cursed-princess-club', 1537),
             cls('Cyberbunk', 'sf/cyberbunk', 466),
             cls('Cyberforce', 'super-hero/cyberforce', 531),
+            cls('CydoniaShattering', 'fantasy/cydonia-shattering', 2881),
             cls('CykoKO', 'super-hero/cyko-ko', 560),
             cls('Darbi', 'action/darbi', 1098),
             cls('Darchon', 'challenge/darchon', 532053),
@@ -149,6 +156,8 @@ class WebToons(ParserScraper):
             cls('DrawnToYou', 'challenge/drawn-to-you', 172022),
             cls('DrFrost', 'drama/dr-frost', 371),
             cls('DuelIdentity', 'challenge/duel-identity', 532064),
+            cls('DungeonCleaningLife', 'action/the-dungeon-cleaning-life-of-a-once-genius-hunter', 4677),
+            cls('DungeonsAndDoodlesTalesFromTheTables', 'canvas/dungeons-doodles-tales-from-the-tables', 682646),
             cls('DungeonMinis', 'challenge/dungeonminis', 64132),
             cls('Dustinteractive', 'comedy/dustinteractive', 907),
             cls('DutyAfterSchool', 'sf/duty-after-school', 370),
@@ -163,9 +172,11 @@ class WebToons(ParserScraper):
             cls('EpicV', 'comedy/epic-v', 353),
             cls('EscapeRoom', 'thriller/escape-room', 1815),
             cls('EverywhereAndNowhere', 'comedy/everywhere-and-nowhere', 1598),
+            cls('FalseKnees', 'canvas/false-knees', 79544),
             cls('FAMILYMAN', 'drama/family-man', 85),
             cls('FantasySketchTheGame', 'sf/fantasy-sketch', 1020),
             cls('Faust', 'supernatural/faust', 522),
+            cls('FinalRaidBoss', 'fantasy/the-final-raid-boss', 3921),
             cls('FINALITY', 'mystery/finality', 1457),
             cls('Firebrand', 'supernatural/firebrand', 877),
             cls('FirstDefense', 'challenge/first-defense', 532072),
@@ -200,11 +211,13 @@ class WebToons(ParserScraper):
             cls('HeliosFemina', 'fantasy/helios-femina', 638),
             cls('HelloWorld', 'slice-of-life/hello-world', 827),
             cls('Hellper', 'fantasy/hellper', 185),
+            cls('Hench', 'canvas/hench/', 857225),
             cls('HeroineChic', 'super-hero/heroine-chic', 561),
             cls('HIVE', 'thriller/hive', 65),
             cls('Hooky', 'fantasy/hooky', 425),
             cls('HoovesOfDeath', 'fantasy/hooves-of-death', 1535),
             cls('HouseOfStars', 'fantasy/house-of-stars', 1620),
+            cls('HowToBeAMindReaver', 'canvas/how-to-be-a-mind-reaver', 301213),
             cls('HowToBecomeADragon', 'fantasy/how-to-become-a-dragon', 1973),
             cls('HowToLove', 'slice-of-life/how-to-love', 472),
             cls('IDontWantThisKindOfHero', 'super-hero/i-dont-want-this-kind-of-hero', 98),
@@ -220,6 +233,7 @@ class WebToons(ParserScraper):
             cls('ItsMine', 'drama/its-mine', 2010),
             cls('JackieRose', 'supernatural/jackie-rose', 613),
             cls('JingleJungle', 'slice-of-life/jingle-jungle', 282),
+            cls('JustACat', 'fantasy/just-a-cat', 7633),
             cls('JustAskYuli', 'slice-of-life/just-ask-yuli', 402),
             cls('JustForKicks', 'slice-of-life/just-for-kicks', 1152),
             cls('JustFriends', 'challenge/just-friends', 190722),
@@ -231,6 +245,7 @@ class WebToons(ParserScraper):
             cls('KindOfLove', 'slice-of-life/kind-of-love', 1850),
             cls('KissItGoodbye', 'challenge/kiss-it-goodbye', 443703),
             cls('KnightRun', 'sf/knight-run', 67),
+            cls('KnightUnderMyHeart', 'action/knight-under-my-heart', 4215),
             cls('Kubera', 'fantasy/kubera', 83),
             cls('LalinsCurse', 'supernatural/lalins-curse', 1601),
             cls('Lars', 'slice-of-life/lars', 358),
@@ -257,6 +272,7 @@ class WebToons(ParserScraper):
             cls('LUMINE', 'fantasy/lumine', 1022),
             cls('Lunarbaboon', 'slice-of-life/lunarbaboon', 523),
             cls('MageAndDemonQueen', 'comedy/mage-and-demon-queen', 1438),
+            cls('MageAndMimic', 'comedy/mage-and-mimic', 5973),
             cls('Magical12thGraders', 'super-hero/magical-12th-graders', 90),
             cls('Magician', 'fantasy/magician', 70),
             cls('MagicSodaPop', 'fantasy/magic-soda-pop', 1947),
@@ -272,6 +288,7 @@ class WebToons(ParserScraper):
             cls('MidnightRain', 'drama/midnight-rain', 1797),
             cls('MidnightRhapsody', 'slice-of-life/midnight-rhapsody', 116),
             cls('MidnightRhapsodySeason2', 'slice-of-life/midnight-rhapsody-season2', 365),
+            cls('Miez', 'sf/miez', 2719),
             cls('MissAbbottAndTheDoctor', 'romance/miss-abbott-and-the-doctor', 707),
             cls('MonsterIsle', 'challenge/monster-isle', 531999),
             cls('MOONBEARD', 'comedy/moon-beard', 471),
@@ -287,25 +304,27 @@ class WebToons(ParserScraper):
             cls('MyGiantNerdBoyfriend', 'slice-of-life/my-giant-nerd-boyfriend', 958),
             cls('MyKittyAndOldDog', 'slice-of-life/my-kitty-and-old-dog', 184),
             cls('MyNameIsBenny', 'slice-of-life/my-name-is-benny', 1279),
+            cls('MySClassHunter', 'action/my-s-class-hunters', 3963),
+            cls('MythicItemObtained', 'fantasy/mythic-item-obtained', 4582),
             cls('MyWallflowerKiss', 'challenge/my-wallflower-kiss', 151869),
+            cls('MyWifeIsAHalfDragon', 'canvas/my-wife-is-a-half-dragon', 1013669),
             cls('NanoList', 'sf/nano-list', 700),
             cls('NationalDogDay2016', 'slice-of-life/national-dog-day', 747),
             cls('NewLifeProject', 'comedy/new-life-project', 279),
             cls('Newman', 'fantasy/newman', 405),
             cls('NewNormalClass8', 'drama/new-normal-class-8', 100),
             cls('Nicholalala', 'slice-of-life/nicholalala', 418),
-            cls('NightmareFactory', 'thriller/nightmare-factory', 616),
             cls('Noblesse', 'action/noblesse', 87),
             cls('NoblesseRaisAdventure', 'action/noblesse-spin-off', 608),
             cls('NoScope', 'sports/no-scope', 1572),
             cls('NotEvenBones', 'thriller/not-even-bones', 1756),
             cls('NothingSpecial', 'fantasy/nothing-special', 1188),
+            cls('NotSoLucky', 'challenge/not-so-lucky', 673387),
             cls('OddGirlOut', 'drama/odd-girl-out', 1420),
             cls('OhHoly', 'romance/oh-holy', 809),
             cls('OmniscientReader', 'action/omniscient-reader', 2154),
             cls('ORANGEMARMALADE', 'romance/orange-marmalade', 97),
             cls('Outrage', 'super-hero/outrage', 1450),
-            cls('OVERPOWERED', 'challenge/overpowered', 85292),
             cls('PacificRimAmara', 'sf/pacific-rim-amara', 1327),
             cls('PandorasBlogs', 'challenge/pandoras-blogs', 532007),
             cls('PaperRoses', 'challenge/paper-roses', 39736),
@@ -393,6 +412,7 @@ class WebToons(ParserScraper):
             cls('TheGreenhouse', 'challenge/the-greenhouse-gl', 278312),
             cls('TheKissBet', 'romance/the-kiss-bet', 1617),
             cls('TheLifeOfTheThreeBears', 'slice-of-life/the-life-of-the-three-bears', 390),
+            cls('TheLittleTrashmaid', 'canvas/the-little-trashmaid', 300138),
             cls('ThePurpleHeart', 'super-hero/the-purple-heart', 723),
             cls('TheRedBook', 'horror/the-red-book', 467),
             cls('TheRedHook', 'super-hero/the-red-hook', 643),
@@ -400,6 +420,7 @@ class WebToons(ParserScraper):
             cls('TheRoomies', 'challenge/the-roomies-archive', 513669),
             cls('TheShadowProphet', 'drama/the-shadow-prophet', 1881),
             cls('TheSoundOfYourHeart', 'comedy/the-sound-of-your-heart', 269),
+            cls('TheSpectrumOfUs', 'challenge/the-spectrum-of-us', 334525),
             cls('TheSteamDragonExpress', 'fantasy/steam-dragon-express', 1270),
             cls('TheStoriesOfThoseAroundMe', 'romance/the-stories-of-those-around-me', 96),
             cls('TheStrangeTalesOfOscarZahn', 'fantasy/the-strange-tales-of-oscar-zahn', 685),
@@ -407,6 +428,7 @@ class WebToons(ParserScraper):
             cls('TheWeeklyRoll', 'challenge/the-weekly-roll', 358889),
             cls('TheWeightOfOurSky', 'historical/the-weight-of-our-sky', 1739),
             cls('TheWitchAndTheBull', 'fantasy/the-witch-and-the-bull', 1892),
+            cls('TheWolfAndRedRidingHood', 'comedy/wolf-and-red-riding-hood', 2142),
             cls('TheWolfmanOfWulvershire', 'mystery/the-wolfman-of-wulvershire', 1784),
             cls('TheWorldWhereIBelong', 'supernatural/the-world-where-i-belong', 1318),
             cls('TheWrathAndTheDawn', 'fantasy/the-wrath-and-the-dawn', 1772),
@@ -433,6 +455,7 @@ class WebToons(ParserScraper):
             cls('UpAndOut', 'slice-of-life/up-and-out', 488),
             cls('UrbanAnimal', 'super-hero/urban-animal', 1483),
             cls('Uriah', 'horror/uriah', 1607),
+            cls('VampireFamily', 'comedy/vampire-family', 6402),
             cls('VarsityNoir', 'mystery/varsity-noir', 1613),
             cls('VersionDayAndNight', 'drama/version-day-and-night', 1796),
             cls('WafflesAndPancakes', 'slice-of-life/waffles-and-pancakes', 1310),
@@ -452,6 +475,7 @@ class WebToons(ParserScraper):
             cls('WitchCreekRoad', 'horror/witch-creek-road', 1453),
             cls('WitchHunt', 'supernatural/witch-hunt', 363),
             cls('Wolfsbane', 'horror/wolfsbane', 1826),
+            cls('WorldsStrongestTroll', 'fantasy/worlds-strongest-troll', 5315),
             cls('XINK3R', 'super-hero/xinker', 541),
             cls('YourAdventure', 'comedy/your-adventure', 506),
             cls('YourLetter', 'drama/your-letter', 1540),
