@@ -9,11 +9,10 @@ set -eu
 HOME="/tmp/home"
 mkdir -p "$HOME"
 cd /work
-pip install --no-warn-script-location --user pytest-cov PySocks
-pip install --no-warn-script-location --user -e '.[css,dev]'
+pip install --no-warn-script-location --user PySocks
+pip install --no-warn-script-location --user -e '.[compression,dev]'
 
-TESTALL=1 python3 -m py.test -v --cov=. --cov-report xml \
+TESTALL=1 python3 -m pytest -v --cov=. --cov-report xml \
     --alluredir=allure-data \
     --tb=short -n10 --junitxml=junit.xml \
     tests/modules/check_comics.py || true
-
