@@ -1,9 +1,9 @@
 # SPDX-License-Identifier: MIT
-# Copyright (C) 2019 Tobias Gruetzmacher
+# SPDX-FileCopyrightText: © 2019 Tobias Gruetzmacher
 from dosagelib.helpers import joinPathPartsNamer, queryNamer
 
 
-class TestNamer(object):
+class TestNamer:
     """
     Tests for comic namer.
     """
@@ -16,6 +16,8 @@ class TestNamer(object):
     def test_joinPathPartsNamer(self):
         imgurl = 'https://HOST/wp-content/uploads/2019/02/tennis5wp-1.png'
         pageurl = 'https://HOST/2019/03/11/12450/'
-        assert joinPathPartsNamer((0, 1, 2))(self, imgurl, pageurl) == '2019_03_11_tennis5wp-1.png'
-        assert joinPathPartsNamer((0, 1, 2), (-1,), '-')(self, imgurl, pageurl) == '2019-03-11-tennis5wp-1.png'
-        assert joinPathPartsNamer((0, -2), ())(self, imgurl, pageurl) == '2019_12450'
+        assert joinPathPartsNamer(pageparts=(0, 1, 2), imageparts=(-1,))(self,
+            imgurl, pageurl) == '2019_03_11_tennis5wp-1.png'
+        assert joinPathPartsNamer(pageparts=(0, 1, 2), imageparts=(-1,), joinchar='-')(self,
+            imgurl, pageurl) == '2019-03-11-tennis5wp-1.png'
+        assert joinPathPartsNamer(pageparts=(0, -1))(self, imgurl, pageurl) == '2019_12450'
