@@ -1,5 +1,5 @@
 # SPDX-License-Identifier: MIT
-# Copyright (C) 2020 Tobias Gruetzmacher
+# SPDX-FileCopyrightText: © 2020 Tobias Gruetzmacher
 
 from lxml import html
 
@@ -21,5 +21,8 @@ class TestXML:
         assert len(self.xpath('//li[d:class("menu-item-object-custom")]')) == 2
         assert len(self.xpath('//li[d:class("menu-item")]')) == 25
 
+    def test_class_start_ext(self):
+        assert len(self.xpath('//li[d:class_start("menu-item-5")]')) == 5
+
     def test_re_ext(self):
         assert len(self.xpath(r'//img[re:test(@src, "posters.*jpg")]')) == 1
